"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.trModel = exports.setLanguageByName = exports.createTranslatedWarningValidator = exports.createTranslatedErrorValidator = void 0;
var _translations = require("./translations");
var _translate = require("./translate");
var _utils = require("./utils");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
var defLangs = {
  en: _translations.en,
  es: _translations.es
};

/**
 * Fully translates objects returned by validators
 *
 * @param {object} msgs Object with errors
 *    or warnings.
 * @returns {object} Messages translated for
 *    use with ReduxForm.
 */
var translateMsgs = function translateMsgs(msgs) {
  var fields = Object.keys(msgs);
  var starting = Array.isArray(msgs) ? [] : {};
  return fields.reduce(function (obj, field) {
    var result;
    var value = msgs[field];
    if (value) {
      if (_typeof(value) == 'object') {
        if (value._translatable) result = (0, _translate.tr)(value.msg, value.msgData);else result = translateMsgs(value);
      } else result = value;
      obj[field] = result;
    }
    return obj;
  }, starting);
};

/**
 * Creates a validator that automatically translates
 * the messages.
 *
 * @param {validator} validator Validator
 * @returns {object}
 */
var createTranslatedValidator = function createTranslatedValidator(validator) {
  return function (model, props) {
    var msgs = validator(model, props);
    return translateMsgs(msgs);
  };
};

/**
 * Creates a validator that automatically translates
 * errors.
 *
 * @param {Schema} schema Schema instance
 * @returns {validator}
 */
var createTranslatedErrorValidator = exports.createTranslatedErrorValidator = function createTranslatedErrorValidator(schema) {
  return createTranslatedValidator(schema.validators.error);
};

/**
 * Creates a validator that automatically translates
 * warnings.
 *
 * @param {Schema} schema Schema instance
 * @returns {validator}
 */
var createTranslatedWarningValidator = exports.createTranslatedWarningValidator = function createTranslatedWarningValidator(schema) {
  return createTranslatedValidator(schema.validators.warning);
};

/**
 * Loads a language from the languages table.
 *
 * @param {string} name Language code as in 'en' or 'fr'.
 */
var setLanguageByName = exports.setLanguageByName = function setLanguageByName(name) {
  if (name in defLangs) (0, _translate.setLanguage)(defLangs[name]);
};

/**
 * Multipurpose semantic-ish translation.
 *
 * @param {string} model Object name, usually what
 *    you pass as the first parameter when you create
 *    the schema.
 * @param {string} field Field name
 * @param {string} op Thing that varies based on
 *    the type.
 */
var trModel = exports.trModel = function trModel(model, field, op) {
  return (0, _translate.tr)((0, _utils.trPath)(model, field, op));
};