"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.trPath = exports.schemaTypeEx = exports.schemaType = exports.objectTraverse = exports.objectIsEmpty = exports.mergeInitialValues = exports.getType = exports.forceArray = exports.arrayTrim = exports.arrLast = void 0;
var _schema = require("./schema");
function _toArray(r) { return _arrayWithHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _arrayWithHoles(r) { if (Array.isArray(r)) return r; }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
var getKind = function getKind(thing) {
  if (Array.isArray(thing)) {
    return 'array';
  } else {
    return _typeof(thing);
  }
};

/**
 * Returns the type of a value. That is
 * 'array' for an array, the name of the
 * schema for a Schema instance or, otherwise,
 * the javascript typeof of the value.
 *
 * @param {any} thing Value
 *
 * @returns {string} Type name.
 */
var getType = exports.getType = function getType(thing) {
  var kind = getKind(thing);
  if (kind == 'object' && thing instanceof _schema.Schema) return thing.getType();else return kind;
};

/**
 * Translates schema specification type. Types can
 * be specified with a string or a constructor like
 * String.
 *
 * @param {string|function} type Type specification.
 *
 * @returns {string} Type as string.
 */
var schemaType = exports.schemaType = function schemaType(type) {
  if (typeof type == 'function') return _typeof(type());else return type;
};

/**
 * Translates the schema's type specification. Type
 * can be specified as with schemaType and also can
 * be a subschema or an array of other schema.
 *
 * @param {any} type Can be:
 *    - String like 'number'
 *    - Constructor like Number
 *    - Schema instance
 *    - Array with schema instance in the first element.
 *        Example: [client]
 */
var schemaTypeEx = exports.schemaTypeEx = function schemaTypeEx(type) {
  if (type instanceof _schema.Schema) return 'schema';else {
    var isArray = Array.isArray(type);
    if (isArray && type.length > 0 && type[0] instanceof _schema.Schema) return 'array';else return schemaType(type);
  }
};

/**
 * If value is not an array, it returns an array
 * with it as the only element.
 *
 * @param {any} thing Value
 *
 * @returns {array} [thing]
 */
var forceArray = exports.forceArray = function forceArray(thing) {
  return Array.isArray(thing) ? thing : [thing];
};

/**
 * Says is an object is empty for the likes of
 * this library.
 *
 * @param {array|object} obj Object or array.
 *
 * @returns {boolean} true if the array is empty or
 *    the object doesn't have keys.
 */
var objectIsEmpty = exports.objectIsEmpty = function objectIsEmpty(obj) {
  if (Array.isArray(obj)) {
    return obj.length == 0;
  } else {
    return _typeof(obj) == 'object' && Object.keys(obj).length === 0 && obj.constructor === Object;
  }
};

/**
 * Removes successful fields from the message
 * arrays used in ReduxForm's FieldArray.
 *
 * @param {array} arr Array with the validation
 *    messages.
 *
 * @returns {array} Pruned array.
 */
var arrayTrim = exports.arrayTrim = function arrayTrim(arr) {
  if (Array.isArray(arr)) {
    return arr.reduceRight(function (arr, msg) {
      if (arr.length == 0 && objectIsEmpty(msg)) return arr;else return [msg].concat(_toConsumableArray(arr));
    }, []);
  } else {
    return arr;
  }
};

/**
 * Returns the last element of an array
 *
 * @params {array} arr Array
 *
 * @returns {any} Last element
 */
var arrLast = exports.arrLast = function arrLast(arr) {
  return arr[arr.length - 1];
};

/**
 * Traverses an object. If a subobject doesn't exist,
 * it will return the default value.
 *
 * @param {object} obj Object to traverse
 * @param {array} path Path to traverse the object
 * @param {any} def Default value if subobject doesn't exist
 */
var objectTraverse = exports.objectTraverse = function objectTraverse(obj, path, def) {
  var _path = _toArray(path),
    next = _path[0],
    rest = _path.slice(1);
  if (next in obj) {
    if (rest.length == 0) return obj[next];else return objectTraverse(obj[next], rest, def);
  } else return def;
};

/**
 * Generates a model translation path.
 *
 * @param {string} model Name of the model (ie: 'client')
 * @param {string} field Name of the field
 * @param {string} op Name of the option or any subthing.
 *
 * @returns {string} id for the translation string
 */
var trPath = exports.trPath = function trPath(model, field, op) {
  if (typeof op == 'undefined') return ['models', model, field].join('.');else return ['models', model, field, op].join('.');
};

/**
 * Merges the (passed or not) initialValues with
 * the defaultValue's found in the schema.
 *
 * @param {object} initialValues (optional) Initial values
 *    passed to the form.
 * @param {Schema} schema Schema to hunt for defaults
 *
 * @returns {object} Merged initial values to simulate
 *    defaults.
 */
var mergeInitialValues = exports.mergeInitialValues = function mergeInitialValues() {
  var initialValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var schema = arguments.length > 1 ? arguments[1] : undefined;
  var schemaDef = schema.getSchema();
  var fields = Object.keys(schemaDef);
  var values = Object.assign({}, initialValues);
  return fields.reduce(function (newDefaults, fieldName) {
    var fieldDef = schemaDef[fieldName];
    if ('defaultValue' in fieldDef && !(fieldName in newDefaults)) newDefaults[fieldName] = fieldDef.defaultValue;
    if (fieldDef.type instanceof _schema.Schema) {
      var instanceCur = newDefaults[fieldName] || {};
      var instanceDefs = mergeInitialValues(instanceCur, fieldDef.type);
      newDefaults[fieldName] = instanceDefs;
    }
    return newDefaults;
  }, values);
};